@@ -41,7 +41,7 @@ copyright = '2008, Georg Brandl'
 #
 # The short X.Y version.
 import sphinx
-version = sphinx.__version__
+version = sphinx.__released__
 # The full version, including alpha/beta/rc tags.
 release = version
 
@@ -131,8 +131,6 @@ latex_logo = '_static/sphinx.png'
 # Documents to append as an appendix to all manuals.
 #latex_appendices = []
 
-automodule_skip_lines = 4
-
 
 # Extension interface
 # -------------------
@@ -153,7 +151,7 @@ def parse_directive(env, sig, signode):
     name, args = m.groups()
     dec_name = '.. %s::' % name
     signode += addnodes.desc_name(dec_name, dec_name)
-    signode += addnodes.desc_classname(args, args)
+    signode += addnodes.desc_addname(args, args)
     return name
 
 
@@ -162,7 +160,27 @@ def parse_role(env, sig, signode):
     return sig
 
 
+event_sig_re = re.compile(r'([a-zA-Z-]+)\s*\((.*)\)')
+
+def parse_event(env, sig, signode):
+    m = event_sig_re.match(sig)
+    if not m:
+        signode += addnodes.desc_name(sig, sig)
+        return sig
+    name, args = m.groups()
+    signode += addnodes.desc_name(name, name)
+    plist = addnodes.desc_parameterlist()
+    for arg in args.split(','):
+        arg = arg.strip()
+        plist += addnodes.desc_parameter(arg, arg)
+    signode += plist
+    return name
+
+
 def setup(app):
+    from sphinx.ext.autodoc import cut_lines
+    app.connect('autodoc-process-docstring', cut_lines(4, what=['module']))
     app.add_description_unit('directive', 'dir', 'pair: %s; directive', parse_directive)
     app.add_description_unit('role', 'role', 'pair: %s; role', parse_role)
     app.add_description_unit('confval', 'confval', 'pair: %s; configuration value')
+    app.add_description_unit('event', 'event', 'pair: %s; event', parse_event)
