@@ -19,8 +19,9 @@ from cStringIO import StringIO
 from sphinx.util import format_exception_cut_frames, save_traceback
 from sphinx.util.console import darkred, nocolor
 
-__revision__ = '$Revision: 64734 $'
-__version__ = '0.4.1'
+__revision__ = '$Revision: 66847 $'
+__version__  = '0.4.3'
+__released__ = '0.4.3'
 
 
 def usage(argv, msg=None):
@@ -29,7 +30,7 @@ def usage(argv, msg=None):
         print >>sys.stderr
     print >>sys.stderr, """\
 Sphinx v%s
-Usage: %s [options] sourcedir outdir [filenames...]"
+Usage: %s [options] sourcedir outdir [filenames...]
 Options: -b <builder> -- builder to use; default is html
          -a        -- write all files; default is to only write new and changed files
          -E        -- don't use a saved environment, always read all files
@@ -98,7 +99,7 @@ def main(argv=sys.argv):
                 return 1
             all_files = True
         elif opt == '-d':
-            doctreedir = val
+            doctreedir = path.abspath(val)
         elif opt == '-c':
             confdir = path.abspath(val)
             if not path.isfile(path.join(confdir, 'conf.py')):
