@@ -49,12 +49,15 @@ class ExtensionError(Exception):
 events = {
     'builder-inited': '',
     'doctree-read' : 'the doctree before being pickled',
-    'doctree-resolved' : 'the doctree, the docname',
+    'doctree-resolved' : 'doctree, docname',
+    'html-page-context': 'pagename, context, doctree or None',
 }
 
+CONFIG_FILENAME = 'conf.py'
+
 class Sphinx(object):
 
-    def __init__(self, srcdir, outdir, doctreedir, buildername,
+    def __init__(self, srcdir, confdir, outdir, doctreedir, buildername,
                  confoverrides, status, warning=sys.stderr, freshenv=False):
         self.next_listener_id = 0
         self._listeners = {}
@@ -62,6 +65,7 @@ class Sphinx(object):
         self.builder = None
 
         self.srcdir = srcdir
+        self.confdir = confdir
         self.outdir = outdir
         self.doctreedir = doctreedir
 
@@ -72,21 +76,17 @@ class Sphinx(object):
         self._events = events.copy()
 
         # read config
-        self.config = Config(srcdir, 'conf.py')
-        if confoverrides:
-            for key, val in confoverrides.items():
-                setattr(self.config, key, val)
+        self.config = Config(confdir, CONFIG_FILENAME, confoverrides)
 
         # load all extension modules
-        for extension in getattr(self.config, 'extensions', ()):
+        for extension in self.config.extensions:
             self.setup_extension(extension)
         # the config file itself can be an extension
-        if hasattr(self.config, 'setup'):
+        if self.config.setup:
             self.config.setup(self)
 
-        # this must happen after loading extension modules, since they
-        # can add custom config values
-        self.config.init_defaults()
+        # now that we know all config values, collect them from conf.py
+        self.config.init_values()
 
         if buildername is None:
             print >>status, 'No builder selected, using default: html'
@@ -157,7 +157,7 @@ class Sphinx(object):
         return listener_id
 
     def disconnect(self, listener_id):
-        for event in self._listeners:
+        for event in self._listeners.itervalues():
             event.pop(listener_id, None)
 
     def emit(self, event, *args):
@@ -221,7 +221,8 @@ class Sphinx(object):
 
 class TemplateBridge(object):
     """
-    
+    This class defines the interface for a "template bridge", that is, a class
+    that renders templates given a template name and a context.
     """
 
     def init(self, builder):
