@@ -25,7 +25,7 @@ from docutils.frontend import OptionParser
 from docutils.readers.doctree import Reader as DoctreeReader
 
 from sphinx import addnodes
-from sphinx.util import ensuredir, relative_uri, SEP, os_path
+from sphinx.util import ensuredir, relative_uri, SEP, os_path, texescape
 from sphinx.htmlhelp import build_hhx
 from sphinx.htmlwriter import HTMLWriter, HTMLTranslator, SmartyPantsHTMLTranslator
 from sphinx.textwriter import TextWriter
@@ -143,6 +143,9 @@ class Builder(object):
                 node['uri'] = candidate
             else:
                 candidate = node['uri']
+            if candidate not in self.env.images:
+                # non-existing URI; let it alone
+                continue
             self.images[candidate] = self.env.images[candidate][1]
 
     # build methods
@@ -211,7 +214,7 @@ class Builder(object):
         warnings = []
         self.env.set_warnfunc(warnings.append)
         self.info(bold('updating environment: '), nonl=1)
-        iterator = self.env.update(self.config, self.app)
+        iterator = self.env.update(self.config, self.srcdir, self.doctreedir, self.app)
         # the first item in the iterator is a summary message
         self.info(iterator.next())
         for docname in self.status_iterator(iterator, 'reading... ', purple):
@@ -377,6 +380,7 @@ class StandaloneHTMLBuilder(Builder):
             docstitle = self.config.html_title,
             shorttitle = self.config.html_short_title,
             show_sphinx = self.config.html_show_sphinx,
+            file_suffix = self.out_suffix,
             rellinks = rellinks,
             builder = self.name,
             parents = [],
@@ -591,7 +595,8 @@ class StandaloneHTMLBuilder(Builder):
                 if path.isfile(fullname):
                     shutil.copyfile(fullname, targetname)
                 elif path.isdir(fullname):
-                    shutil.rmtree(targetname)
+                    if path.exists(targetname):
+                        shutil.rmtree(targetname)
                     shutil.copytree(fullname, targetname)
         # copy logo file (handled differently)
         if self.config.html_logo:
@@ -719,11 +724,12 @@ class PickleHTMLBuilder(StandaloneHTMLBuilder):
         return docname + SEP
 
     def handle_page(self, pagename, ctx, templatename='page.html',
-                    outfilename=None):
+                    outfilename=None, event_arg=None):
         ctx['current_page_name'] = pagename
         sidebarfile = self.config.html_sidebars.get(pagename)
         if sidebarfile:
             ctx['customsidebar'] = sidebarfile
+
         if not outfilename:
             outfilename = path.join(self.outdir, os_path(pagename) + self.out_suffix)
         ensuredir(path.dirname(outfilename))
@@ -803,6 +809,7 @@ class LaTeXBuilder(Builder):
     def init(self):
         self.docnames = []
         self.document_data = []
+        texescape.init()
 
     def get_outdated_docs(self):
         return 'all documents' # for now
@@ -971,6 +978,9 @@ class ChangesBuilder(Builder):
         libchanges = {}
         apichanges = []
         otherchanges = {}
+        if version not in self.env.versionchanges:
+            self.info(bold('no changes in this version.'))
+            return
         self.info(bold('writing summary file...'))
         for type, docname, lineno, module, descname, content in \
                 self.env.versionchanges[version]:
