@@ -63,7 +63,7 @@ default_settings = {
 
 # This is increased every time an environment attribute is added
 # or changed to properly invalidate pickle files.
-ENV_VERSION = 24
+ENV_VERSION = 25
 
 
 default_substitutions = set([
@@ -242,6 +242,7 @@ class BuildEnvironment:
         self.toctree_includes = {}  # docname -> list of toctree includefiles
         self.files_to_rebuild = {}  # docname -> set of files (containing its TOCs)
                                     # to rebuild too
+        self.glob_toctrees = set()  # docnames that have :glob: toctrees
 
         # X-ref target inventory
         self.descrefs = {}          # fullname -> docname, desctype
@@ -296,6 +297,7 @@ class BuildEnvironment:
             self.toctree_includes.pop(docname, None)
             self.filemodules.pop(docname, None)
             self.indexentries.pop(docname, None)
+            self.glob_toctrees.discard(docname)
 
             for subfn, fnset in self.files_to_rebuild.items():
                 fnset.discard(docname)
@@ -396,7 +398,7 @@ class BuildEnvironment:
 
         return added, changed, removed
 
-    def update(self, config, app=None):
+    def update(self, config, srcdir, doctreedir, app=None):
         """(Re-)read all files new or changed since last update.  Yields a summary
         and then docnames as it processes them.  Store all environment docnames
         in the canonical format (ie using SEP as a separator in place of
@@ -416,8 +418,23 @@ class BuildEnvironment:
                     break
             else:
                 msg = ''
+            # this value is not covered by the above loop because it is handled
+            # specially by the config class
+            if self.config.extensions != config.extensions:
+                msg = '[extensions changed] '
+                config_changed = True
+        # the source and doctree directories may have been relocated
+        self.srcdir = srcdir
+        self.doctreedir = doctreedir
         self.find_files(config)
+
         added, changed, removed = self.get_outdated_files(config_changed)
+
+        # if files were added or removed, all documents with globbed toctrees
+        # must be reread
+        if added or removed:
+            changed.update(self.glob_toctrees)
+
         msg += '%s added, %s changed, %s removed' % (len(added), len(changed),
                                                      len(removed))
         yield msg
@@ -544,12 +561,12 @@ class BuildEnvironment:
             imgpath = path.normpath(path.join(docdir, imguri))
             node['uri'] = imgpath
             if imgpath.endswith(os.extsep + '*'):
-                for filename in glob(imgpath):
-                    basename, ext = os.path.splitext(filename)
-                    if ext == '.pdf':
-                        candidates['application/pdf'] = filename
-                    elif ext == '.svg':
-                        candidates['image/svg+xml'] = filename
+                for filename in glob(path.join(self.srcdir, imgpath)):
+                    dir, base = path.split(filename)
+                    if base.lower().endswith('.pdf'):
+                        candidates['application/pdf'] = path.join(docdir, base)
+                    elif base.lower().endswith('.svg'):
+                        candidates['image/svg+xml'] = path.join(docdir, base)
                     else:
                         f = open(filename, 'rb')
                         try:
@@ -557,23 +574,24 @@ class BuildEnvironment:
                         finally:
                             f.close()
                         if imgtype:
-                            candidates['image/' + imgtype] = filename
+                            candidates['image/' + imgtype] = path.join(docdir, base)
             else:
                 candidates['*'] = imgpath
-            for img in candidates.itervalues():
-                self.dependencies.setdefault(docname, set()).add(img)
-                if not os.access(path.join(self.srcdir, img), os.R_OK):
-                    self.warn(docname, 'Image file not readable: %s' % img, node.line)
-                if img in self.images:
-                    self.images[img][0].add(docname)
+            for imgpath in candidates.itervalues():
+                self.dependencies.setdefault(docname, set()).add(imgpath)
+                if not os.access(path.join(self.srcdir, imgpath), os.R_OK):
+                    self.warn(docname, 'Image file not readable: %s' % imgpath,
+                              node.line)
+                if imgpath in self.images:
+                    self.images[imgpath][0].add(docname)
                     continue
-                uniquename = path.basename(img)
+                uniquename = path.basename(imgpath)
                 base, ext = path.splitext(uniquename)
                 i = 0
                 while uniquename in existing_names:
                     i += 1
                     uniquename = '%s%s%s' % (base, i, ext)
-                self.images[img] = (set([docname]), uniquename)
+                self.images[imgpath] = (set([docname]), uniquename)
                 existing_names.add(uniquename)
 
     def process_metadata(self, docname, doctree):
@@ -638,6 +656,8 @@ class BuildEnvironment:
     def note_toctree(self, docname, toctreenode):
         """Note a TOC tree directive in a document and gather information about
            file relations from it."""
+        if toctreenode['glob']:
+            self.glob_toctrees.add(docname)
         includefiles = toctreenode['includefiles']
         for includefile in includefiles:
             # note that if the included file is rebuilt, this one must be
@@ -807,10 +827,15 @@ class BuildEnvironment:
             for includefile in includefiles:
                 try:
                     toc = self.tocs[includefile].deepcopy()
+                    if not toc.children:
+                        # empty toc means: no titles will show up in the toctree
+                        self.warn(docname, 'toctree contains reference to document '
+                                  '%r that doesn\'t have a title: no link will be '
+                                  'generated' % includefile)
                 except KeyError:
                     # this is raised if the included file does not exist
-                    self.warn(docname, 'toctree contains ref to nonexisting '
-                              'file %r' % includefile)
+                    self.warn(docname, 'toctree contains reference to nonexisting '
+                              'document %r' % includefile)
                 else:
                     # if titles_only is given, only keep the main title and
                     # sub-toctrees
