@@ -176,7 +176,7 @@ def get_module_charset(module):
     if module in _module_charsets:
         return _module_charsets[module]
     try:
-        filename = __import__(module, None, None, ['']).__file__
+        filename = __import__(module, None, None, ['foo']).__file__
     except (ImportError, AttributeError):
         return None
     if filename[-4:].lower() in ('.pyc', '.pyo'):
@@ -252,6 +252,8 @@ def format_signature(what, obj):
         # classes without __init__ method?
         if obj is None or obj is object.__init__:
             return ''
+    if inspect.isbuiltin(obj) or inspect.ismethoddescriptor(obj):
+        return ''
     argspec = inspect.getargspec(obj)
     if what in ('class', 'method') and argspec[0] and \
        argspec[0][0] in ('cls', 'self'):
@@ -305,6 +307,7 @@ def generate_rst(what, name, members, options, add_content, document, lineno,
         if path:
             mod_cls = path
         else:
+            mod_cls = None
             # if documenting a class-level object without path, there must be a
             # current class, either from a parent auto directive ...
             if hasattr(env, 'autodoc_current_class'):
@@ -344,10 +347,11 @@ def generate_rst(what, name, members, options, add_content, document, lineno,
             modfile = None  # e.g. for builtin and C modules
         for part in objpath:
             todoc = getattr(todoc, part)
-    except (ImportError, AttributeError):
+    except (ImportError, AttributeError), err:
         warnings.append(document.reporter.warning(
-            'autodoc can\'t import/find %s %r, check your spelling '
-            'and sys.path' % (what, str(fullname)), line=lineno))
+            'autodoc can\'t import/find %s %r, it reported error: "%s",'
+            'please check your spelling and sys.path' %
+            (what, str(fullname), err), line=lineno))
         return warnings, result
 
     # check __module__ of object if wanted (for members not given explicitly)
@@ -397,6 +401,7 @@ def generate_rst(what, name, members, options, add_content, document, lineno,
                      u':class:`%s.%s`' % (b.__module__, b.__name__)
                      for b in todoc.__bases__]
             result.append(indent + u'   Bases: %s' % ', '.join(bases), '<autodoc>')
+            result.append(u'', '<autodoc>')
 
     # the module directive doesn't have content
     if what != 'module':
@@ -548,7 +553,8 @@ def members_option(arg):
 
 def setup(app):
     mod_options = {'members': members_option, 'undoc-members': directives.flag,
-                   'noindex': directives.flag}
+                   'noindex': directives.flag, 'inherited-members': directives.flag,
+                   'show-inheritance': directives.flag}
     cls_options = {'members': members_option, 'undoc-members': directives.flag,
                    'noindex': directives.flag, 'inherited-members': directives.flag,
                    'show-inheritance': directives.flag}
