@@ -14,6 +14,8 @@ import cgi
 import re
 import parser
 
+from sphinx.util.texescape import tex_hl_escape_map
+
 try:
     import pygments
     from pygments import highlight
@@ -56,14 +58,9 @@ else:
         _lexer.add_filter('raiseonerror')
 
 
-
-def escape_tex(text):
-    return text.replace('@', '\x00').    \
-                replace('[', '\x01').    \
-                replace(']', '\x02').    \
-                replace('\x00', '@at[]').\
-                replace('\x01', '@lb[]').\
-                replace('\x02', '@rb[]')
+escape_hl_chars = {ord(u'@'): u'@at[]',
+                   ord(u'['): u'@lb[]',
+                   ord(u']'): u'@rb[]'}
 
 # used if Pygments is not available
 _LATEX_STYLES = r'''
@@ -87,22 +84,31 @@ class PygmentsBridge(object):
             return
         if stylename == 'sphinx':
             style = SphinxStyle
+        elif '.' in stylename:
+            module, stylename = stylename.rsplit('.', 1)
+            style = getattr(__import__(module, None, None, ['']), stylename)
         else:
             style = get_style_by_name(stylename)
         self.hfmter = {False: HtmlFormatter(style=style),
                        True: HtmlFormatter(style=style, linenos=True)}
-        self.lfmter = {False: LatexFormatter(style=style),
-                       True: LatexFormatter(style=style, linenos=True)}
+        self.lfmter = {False: LatexFormatter(style=style, commandprefix='PYG'),
+                       True: LatexFormatter(style=style, linenos=True,
+                                            commandprefix='PYG')}
+
+    def unhighlighted(self, source):
+        if self.dest == 'html':
+            return '<pre>' + cgi.escape(source) + '</pre>\n'
+        else:
+            # first, escape highlighting characters like Pygments does
+            source = source.translate(escape_hl_chars)
+            # then, escape all characters nonrepresentable in LaTeX
+            source = source.translate(tex_hl_escape_map)
+            return '\\begin{Verbatim}[commandchars=@\\[\\]]\n' + \
+                   source + '\\end{Verbatim}\n'
 
     def highlight_block(self, source, lang, linenos=False):
-        def unhighlighted():
-            if self.dest == 'html':
-                return '<pre>' + cgi.escape(source) + '</pre>\n'
-            else:
-                return '\\begin{Verbatim}[commandchars=@\\[\\]]\n' + \
-                       escape_tex(source) + '\\end{Verbatim}\n'
         if not pygments:
-            return unhighlighted()
+            return self.unhighlighted(source)
         if lang == 'python':
             if source.startswith('>>>'):
                 # interactive session
@@ -124,10 +130,17 @@ class PygmentsBridge(object):
                 if sys.version_info >= (2, 5):
                     src = 'from __future__ import with_statement\n' + src
 
+                if isinstance(src, unicode):
+                    # Non-ASCII chars will only occur in string literals
+                    # and comments.  If we wanted to give them to the parser
+                    # correctly, we'd have to find out the correct source
+                    # encoding.  Since it may not even be given in a snippet,
+                    # just replace all non-ASCII characters.
+                    src = src.encode('ascii', 'replace')
                 try:
                     parser.suite(src)
                 except parsing_exceptions:
-                    return unhighlighted()
+                    return self.unhighlighted(source)
                 else:
                     lexer = lexers['python']
         else:
@@ -137,12 +150,15 @@ class PygmentsBridge(object):
                 lexer = lexers[lang] = get_lexer_by_name(lang)
                 lexer.add_filter('raiseonerror')
         try:
-            fmter = (self.dest == 'html' and self.hfmter or self.lfmter)[bool(linenos)]
-            return highlight(source, lexer, fmter)
+            if self.dest == 'html':
+                return highlight(source, lexer, self.hfmter[bool(linenos)])
+            else:
+                hlsource = highlight(source, lexer, self.lfmter[bool(linenos)])
+                return hlsource.translate(tex_hl_escape_map)
         except ErrorToken:
             # this is most probably not the selected language,
             # so let it pass unhighlighted
-            return unhighlighted()
+            return self.unhighlighted(source)
 
     def get_stylesheet(self):
         if not pygments:
