@@ -12,9 +12,13 @@
 import sys, os, time
 from os import path
 
-from sphinx.util import make_filename
-from sphinx.util.console import purple, bold, red, nocolor
+TERM_ENCODING = getattr(sys.stdin, 'encoding', None)
 
+from sphinx.util import make_filename
+from sphinx.util.console import purple, bold, red, turquoise, nocolor
+
+
+PROMPT_PREFIX = '> '
 
 QUICKSTART_CONF = '''\
 # -*- coding: utf-8 -*-
@@ -54,8 +58,8 @@ source_suffix = '%(suffix)s'
 master_doc = '%(master)s'
 
 # General substitutions.
-project = %(project)r
-copyright = '%(year)s, %(author)s'
+project = u'%(project)s'
+copyright = u'%(copyright)s'
 
 # The default replacements for |version| and |release|, also used in various
 # other places throughout the built documents.
@@ -76,7 +80,7 @@ today_fmt = '%%B %%d, %%Y'
 
 # List of directories, relative to source directories, that shouldn't be searched
 # for source files.
-#exclude_dirs = []
+exclude_trees = [%(exclude_trees)s]
 
 # The reST default role (used for this markup: `text`) to use for all documents.
 #default_role = None
@@ -111,8 +115,8 @@ html_style = 'default.css'
 # A shorter title for the navigation bar.  Default is the same as html_title.
 #html_short_title = None
 
-# The name of an image file (within the static path) to place at the top of
-# the sidebar.
+# The name of an image file (relative to this directory) to place at the top
+# of the sidebar.
 #html_logo = None
 
 # The name of an image file (within the static path) to use as favicon of the
@@ -176,7 +180,8 @@ htmlhelp_basename = '%(project_fn)sdoc'
 # Grouping the document tree into LaTeX files. List of tuples
 # (source start file, target name, title, author, document class [howto/manual]).
 latex_documents = [
-  ('%(master)s', '%(project_fn)s.tex', '%(project)s Documentation', '%(author)s', 'manual'),
+  ('%(master)s', '%(project_fn)s.tex', u'%(project_doc)s',
+   u'%(author)s', 'manual'),
 ]
 
 # The name of an image file (relative to this directory) to place at the top of
@@ -231,7 +236,8 @@ PAPER         =
 # Internal variables.
 PAPEROPT_a4     = -D latex_paper_size=a4
 PAPEROPT_letter = -D latex_paper_size=letter
-ALLSPHINXOPTS   = -d %(rbuilddir)s/doctrees $(PAPEROPT_$(PAPER)) $(SPHINXOPTS) %(rsrcdir)s
+ALLSPHINXOPTS   = -d %(rbuilddir)s/doctrees $(PAPEROPT_$(PAPER)) \
+$(SPHINXOPTS) %(rsrcdir)s
 
 .PHONY: help clean html web pickle htmlhelp latex changes linkcheck
 
@@ -328,14 +334,24 @@ def ok(x):
 def do_prompt(d, key, text, default=None, validator=nonempty):
     while True:
         if default:
-            prompt = purple('> %s [%s]: ' % (text, default))
+            prompt = purple(PROMPT_PREFIX + '%s [%s]: ' % (text, default))
         else:
-            prompt = purple('> ' + text + ': ')
+            prompt = purple(PROMPT_PREFIX + text + ': ')
         x = raw_input(prompt)
         if default and not x:
             x = default
+        if x.decode('ascii', 'replace').encode('ascii', 'replace') != x:
+            if TERM_ENCODING:
+                x = x.decode(TERM_ENCODING)
+            else:
+                print turquoise('* Note: non-ASCII characters entered and terminal '
+                                'encoding unknown -- assuming UTF-8 or Latin-1.')
+                try:
+                    x = x.decode('utf-8')
+                except UnicodeDecodeError:
+                    x = x.decode('latin1')
         if validator and not validator(x):
-            print red(" * " + validator.__doc__)
+            print red('* ' + validator.__doc__)
             continue
         break
     d[key] = x
@@ -359,7 +375,7 @@ Enter the root path for documentation.'''
 You have two options for placing the build directory for Sphinx output.
 Either, you use a directory ".build" within the root path, or you separate
 "source" and "build" directories within the root path.'''
-    do_prompt(d, 'sep', 'Separate source and build directories (y/n)', 'n',
+    do_prompt(d, 'sep', 'Separate source and build directories (y/N)', 'n',
               boolean)
     print '''
 Inside the root directory, two more directories will be created; ".templates"
@@ -394,23 +410,24 @@ document is a custom template, you can also set this to another filename.'''
     print '''
 Please indicate if you want to use one of the following Sphinx extensions:'''
     do_prompt(d, 'ext_autodoc', 'autodoc: automatically insert docstrings '
-              'from modules (y/n)', 'n', boolean)
+              'from modules (y/N)', 'n', boolean)
     do_prompt(d, 'ext_doctest', 'doctest: automatically test code snippets '
-              'in doctest blocks (y/n)', 'n', boolean)
+              'in doctest blocks (y/N)', 'n', boolean)
     print '''
 If you are under Unix, a Makefile can be generated for you so that you
 only have to run e.g. `make html' instead of invoking sphinx-build
 directly.'''
-    do_prompt(d, 'makefile', 'Create Makefile? (y/n)',
+    do_prompt(d, 'makefile', 'Create Makefile? (Y/n)',
               os.name == 'posix' and 'y' or 'n', boolean)
 
     d['project_fn'] = make_filename(d['project'])
-    d['year'] = time.strftime('%Y')
     d['now'] = time.asctime()
     d['underline'] = len(d['project']) * '='
     d['extensions'] = ', '.join(
         repr('sphinx.ext.' + name) for name in ('autodoc', 'doctest')
         if d['ext_' + name].upper() in ('Y', 'YES'))
+    d['copyright'] = time.strftime('%Y') + ', ' + d['author']
+    d['project_doc'] = d['project'] + ' Documentation'
 
     if not path.isdir(d['path']):
         mkdir_p(d['path'])
@@ -421,19 +438,21 @@ directly.'''
     mkdir_p(srcdir)
     if separate:
         builddir = path.join(d['path'], 'build')
+        d['exclude_trees'] = ''
     else:
         builddir = path.join(srcdir, d['dot'] + 'build')
+        d['exclude_trees'] = repr(d['dot'] + 'build')
     mkdir_p(builddir)
     mkdir_p(path.join(srcdir, d['dot'] + 'templates'))
     mkdir_p(path.join(srcdir, d['dot'] + 'static'))
 
     f = open(path.join(srcdir, 'conf.py'), 'w')
-    f.write(QUICKSTART_CONF % d)
+    f.write((QUICKSTART_CONF % d).encode('utf-8'))
     f.close()
 
     masterfile = path.join(srcdir, d['master'] + d['suffix'])
     f = open(masterfile, 'w')
-    f.write(MASTER_FILE % d)
+    f.write((MASTER_FILE % d).encode('utf-8'))
     f.close()
 
     create_makefile = d['makefile'].upper() in ('Y', 'YES')
@@ -441,7 +460,7 @@ directly.'''
         d['rsrcdir'] = separate and 'source' or '.'
         d['rbuilddir'] = separate and 'build' or d['dot'] + 'build'
         f = open(path.join(d['path'], 'Makefile'), 'w')
-        f.write(MAKEFILE % d)
+        f.write((MAKEFILE % d).encode('utf-8'))
         f.close()
 
     print
