@@ -250,13 +250,13 @@ def patfilter(names, pat):
     Adapted from fnmatch module.
     """
     result = []
-    if not pat in _pat_cache:
+    if pat not in _pat_cache:
         _pat_cache[pat] = re.compile(_translate_pattern(pat))
     match = _pat_cache[pat].match
     return filter(match, names)
 
 
-no_fn_re = re.compile(r'[:/\\?*%|"\'<>. \t]')
+no_fn_re = re.compile(r'[^a-zA-Z0-9_-]')
 
 def make_filename(string):
     return no_fn_re.sub('', string)
