@@ -15,9 +15,9 @@ from util import *
 from sphinx.application import ExtensionError
 
 
-def test_core_config():
-    overrides = {'master_doc': 'master', 'nonexisting_value': 'True'}
-    cfg = TestApp(confoverrides=overrides).config
+@with_app(confoverrides={'master_doc': 'master', 'nonexisting_value': 'True'})
+def test_core_config(app):
+    cfg = app.config
 
     # simple values
     assert 'project' in cfg.__dict__
@@ -43,6 +43,7 @@ def test_core_config():
     # "contains" gives True both for set and unset values
     assert 'project' in cfg
     assert 'html_title' in cfg
+    assert 'nonexisting_value' not in cfg
 
     # invalid values
     raises(AttributeError, getattr, cfg, '_value')
@@ -60,8 +61,8 @@ def test_core_config():
     assert cfg['project'] == cfg.project == 'Sphinx Tests'
 
 
-def test_extension_values():
-    app = TestApp()
+@with_app()
+def test_extension_values(app):
     cfg = app.config
 
     # default value
