@@ -8,21 +8,32 @@
 """
 
 import sys
+import os
 import StringIO
 import tempfile
+import shutil
+
+from functools import wraps
 
 from sphinx import application, builder
 
 from path import path
 
+from nose import tools
+
 
 __all__ = [
+    'test_root',
     'raises', 'raises_msg',
-    'ErrorOutput', 'TestApp',
-    'with_tempdir', 'write_file',
+    'ListOutput', 'TestApp', 'with_app',
+    'path', 'with_tempdir', 'write_file',
+    'sprint',
 ]
 
 
+test_root = path(__file__).parent.joinpath('root').abspath()
+
+
 def _excstr(exc):
     if type(exc) is tuple:
         return str(tuple(map(_excstr, exc)))
@@ -49,21 +60,25 @@ def raises_msg(exc, msg, func, *args, **kwds):
     try:
         func(*args, **kwds)
     except exc, err:
-        assert msg in str(err)
+        assert msg in str(err), "\"%s\" not in \"%s\"" % (msg, err)
     else:
         raise AssertionError('%s did not raise %s' %
                              (func.__name__, _excstr(exc)))
 
 
-class ErrorOutput(object):
+class ListOutput(object):
     """
-    File-like object that raises :exc:`AssertionError` on ``write()``.
+    File-like object that collects written text in a list.
     """
     def __init__(self, name):
         self.name = name
+        self.content = []
+
+    def reset(self):
+        del self.content[:]
 
     def write(self, text):
-        assert False, 'tried to write %r to %s' % (text, self.name)
+        self.content.append(text)
 
 
 class TestApp(application.Sphinx):
@@ -74,33 +89,68 @@ class TestApp(application.Sphinx):
 
     def __init__(self, srcdir=None, confdir=None, outdir=None, doctreedir=None,
                  buildername='html', confoverrides=None, status=None, warning=None,
-                 freshenv=None, confname='conf.py'):
+                 freshenv=None, confname='conf.py', cleanenv=False):
 
         application.CONFIG_FILENAME = confname
 
+        self.cleanup_trees = []
+
         if srcdir is None:
-            srcdir = path(__file__).parent.joinpath('root').abspath()
+            srcdir = test_root
+        if srcdir == '(temp)':
+            tempdir = path(tempfile.mkdtemp())
+            self.cleanup_trees.append(tempdir)
+            temproot = tempdir / 'root'
+            test_root.copytree(temproot)
+            srcdir = temproot
         else:
             srcdir = path(srcdir)
+        self.builddir = srcdir.joinpath('_build')
         if confdir is None:
             confdir = srcdir
         if outdir is None:
-            outdir = srcdir.joinpath('_build', buildername)
+            outdir = srcdir.joinpath(self.builddir, buildername)
+            if not outdir.isdir():
+                outdir.makedirs()
+            self.cleanup_trees.insert(0, outdir)
         if doctreedir is None:
-            doctreedir = srcdir.joinpath(srcdir, '_build', 'doctrees')
+            doctreedir = srcdir.joinpath(srcdir, self.builddir, 'doctrees')
+            if cleanenv:
+                self.cleanup_trees.insert(0, doctreedir)
         if confoverrides is None:
             confoverrides = {}
         if status is None:
             status = StringIO.StringIO()
         if warning is None:
-            warning = ErrorOutput('stderr')
+            warning = ListOutput('stderr')
         if freshenv is None:
-            freshenv = True
+            freshenv = False
 
         application.Sphinx.__init__(self, srcdir, confdir, outdir, doctreedir,
                                     buildername, confoverrides, status, warning,
                                     freshenv)
 
+    def cleanup(self, doctrees=False):
+        for tree in self.cleanup_trees:
+            shutil.rmtree(tree, True)
+
+
+def with_app(*args, **kwargs):
+    """
+    Make a TestApp with args and kwargs, pass it to the test and clean up
+    properly.
+    """
+    def generator(func):
+        @wraps(func)
+        def deco(*args2, **kwargs2):
+            app = TestApp(*args, **kwargs)
+            try:
+                func(app, *args2, **kwargs2)
+            finally:
+                app.cleanup()
+        return deco
+    return generator
+
 
 def with_tempdir(func):
     def new_func():
@@ -115,3 +165,7 @@ def write_file(name, contents):
     f = open(str(name), 'wb')
     f.write(contents)
     f.close()
+
+
+def sprint(*args):
+    sys.stderr.write(' '.join(map(str, args)) + '\n')
